from epubhv import EPUBHV, list_all_epub_in_dir
from pathlib import Path

import argparse
import os


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument("epub", help="file or dir that contains epub files to change")
    parser.add_argument(
        "--v",
        dest="v",
        action="store_true",
        help="change all the epub files to vertical",
    )
    parser.add_argument(
        "--h",
        dest="h",
        action="store_true",
        help="change all the epub files to hortical",
    )

    parser.add_argument(
        "--convert",
        dest="convert",
        choices=[
            "s2t",
            "t2s",
            "s2tw",
            "tw2s",
            "s2hk",
            "hk2s",
            "s2twp",
            "tw2sp",
            "t2tw",
            "hk2t",
            "t2hk",
            "t2jp",
            "jp2t",
            "tw2t",
        ],
        help="""
change all the epub files to specific language


s2t: Simplified Chinese to Traditional Chinese
t2s: Traditional Chinese to Simplified Chinese
s2tw: Simplified Chinese to Traditional Chinese (Taiwan Standard)
tw2s: Traditional Chinese (Taiwan Standard) to Simplified Chinese
s2hk: Simplified Chinese to Traditional Chinese (Hong Kong variant)
hk2s: Traditional Chinese (Hong Kong variant) to Simplified Chinese
s2twp: Simplified Chinese to Traditional Chinese (Taiwan variant)
tw2sp: Traditional Chinese (Taiwan variant) to Simplified Chinese
t2tw: Traditional Chinese (OpenCC Standard) to Taiwan Standard
hk2t: Traditional Chinese (Hong Kong variant) to Traditional Chinese
t2hk: Traditional Chinese (OpenCC Standard) to Hong Kong variant
t2jp: Traditional Chinese Characters (Kyūjitai) to New Japanese Kanji
jp2t: New Japanese Kanji to Traditional Chinese Characters (Kyūjitai)
tw2t: Traditional Chinese (OpenCC Standard) to Traditional Chinese (Taiwan standard)
        """,
    )

    options = parser.parse_args()
    epub_files = Path(options.epub)
    # default is to to_vertical
    method = "to_vertical"
    if options.h:
        method = "to_horizontal"
    elif options.v:
        method = "to_vertical"
    if epub_files.exists():
        if epub_files.is_dir():
            files = list_all_epub_in_dir(epub_files)
            for f in files:
                print(f"{str(f)} is {method}")
                try:
                    e = EPUBHV(f, convert_to=options.convert)
                    e.run(method)
                except Exception as e:
                    print(f"{str(f)} {method} is failed by {str(e)}")
        else:
            print(f"{str(epub_files)} is {method}")
            e = EPUBHV(epub_files, convert_to=options.convert)
            e.run(method)
    else:
        raise Exception("Please make sure it is a dir contains epub or is a epub file.")


if __name__ == "__main__":
    main()



import os
import shutil
from pathlib import Path

from epubhv import EPUBHV, _make_epub_files_dict, list_all_epub_in_dir


def test_find_epub_books():
    assert sorted(list_all_epub_in_dir("tests/test_epub")) == sorted(
        [
            "tests/test_epub/animal_farm.epub",
            "tests/test_epub/Liber_Esther.epub",
            "tests/test_epub/books/lemo.epub",
            "tests/test_epub/sanguo.epub",
        ]
    )


def test_extract_epub_path():
    b = EPUBHV(Path("tests/test_epub/animal_farm.epub"))
    b.extract_one_epub_to_dir()
    assert os.path.exists(".epub_temp_dir/animal_farm")
    shutil.rmtree(".epub_temp_dir")


def test_make_files_dict():
    b = EPUBHV("tests/test_epub/animal_farm.epub")
    b.extract_one_epub_to_dir()
    d = dict(_make_epub_files_dict(".epub_temp_dir/animal_farm"))
    assert sorted(
        [".html", ".css", ".xhtml", "", ".opf", ".ncx", ".jpg", ".xml"]
    ) == sorted(list(d.keys()))
    assert 19 == len(d.get(".html", 0))
    shutil.rmtree(".epub_temp_dir")


def test_change_epub_to_vertical():
    b = EPUBHV("tests/test_epub/animal_farm.epub")
    b.run()
    assert b.opf_file == Path(".epub_temp_dir/animal_farm/content.opf")
    os.remove("animal_farm-v-original.epub")


def test_find_epub_css_files():
    b = EPUBHV("tests/test_epub/animal_farm.epub")
    b._make_epub_values()
    assert b.has_css_file == False
    b.run()
    os.remove("animal_farm-v-original.epub")
    f = EPUBHV("tests/test_epub/books/lemo.epub")
    f.run("to_horizontal")
    assert os.path.exists("lemo-h-original.epub") is True
    os.remove("lemo-h-original.epub")


def test_change_epub_covert():
    f = EPUBHV("tests/test_epub/sanguo.epub", "s2t")
    f.run("to_vertical")
    assert os.path.exists("sanguo-v-s2t.epub") is True
    q = EPUBHV("sanguo-v-s2t.epub")
    q.extract_one_epub_to_dir()
    q._make_epub_values()
    has_t_count = 0
    for html_file in (
        q.files_dict.get(".html", [])
        + q.files_dict.get(".xhtml", [])
        + q.files_dict.get(".htm", [])
    ):
        with open(html_file, "r", encoding="utf-8", errors="ignore") as f:
            r = f.read()
            if r.find("滾滾長江東逝水") > 0:
                has_t_count += 1
    assert has_t_count > 0
    q.run("to_vertical")
    os.remove("sanguo-v-s2t-v-original.epub")



from setuptools import setup

setup(
    name="epubhv",
    author="yihong0618",
    author_email="zouzou0208@gmail.com",
    url="https://github.com/yihong0618/epubhv",
    license="MIT",
    version="0.2.2",
    install_requires=[
        "bs4",
        "lxml",
        "cssutils",
        "opencc-python-reimplemented",
        "soupsieve",
    ],
    entry_points={
        "console_scripts": ["epubhv = epubhv.cli:main"],
    },
)



"""
Follow these steps to change epub books to vertical or horizontal.

"""
import logging
import os
import shutil
import zipfile
from collections import defaultdict
from pathlib import Path

import cssutils
import opencc
from bs4 import BeautifulSoup as bs

cssutils.log.setLevel(logging.CRITICAL)

WRITING_KEY_LIST = ["writing-mode", "-webkit-writing-mode", "-epub-writing-mode"]
V_STYLE_LINE = '<link rel="stylesheet" href="../Style/style.css" type="text/css" />'
# same as v
H_STYLE_LINE = '<link rel="stylesheet" href="../Style/style.css" type="text/css" />'
V_STYLE_LINE_IN_OPF = '<meta content="vertical-rl" name="primary-writing-mode"/>'
H_STYLE_LINE_IN_OPF = '<meta content="horizontal-lr" name="primary-writing-mode"/>'
V_ITEM_TO_ADD_IN_MANIFEST = (
    '<item id="stylesheet" href="Style/style.css" media-type="text/css" />'
)
# same as v
H_ITEM_TO_ADD_IN_MANIFEST = (
    '<item id="stylesheet" href="Style/style.css" media-type="text/css" />'
)


def list_all_epub_in_dir(path):
    files = []
    for root, _, filenames in os.walk(path):
        for filename in filenames:
            files.append(os.path.join(root, filename))
    return files


def _make_epub_files_dict(dir_path):
    files_dict = defaultdict(list)
    for root, _, filenames in os.walk(dir_path):
        for filename in filenames:
            files_dict[Path(filename).suffix].append(Path(root) / Path(filename))
    return files_dict


def load_opf_meta_data(opf_file):
    with open(opf_file, encoding="utf-8", errors="ignore") as f:
        content = f.read()
        soup = bs(content, "xml")
    return soup


class EPUBHV:
    def __init__(self, file_name, convert_to=None):
        self.epub_file = Path(file_name)
        self.has_css_file = False
        self.files_dict = {}
        self.book_path = None
        self.book_name = None
        self.opf_file = None
        if convert_to is not None:
            self.converter = opencc.OpenCC(convert_to)
            self.convert_to = convert_to
        else:
            self.converter = None
            self.convert_to = None

    def extract_one_epub_to_dir(self):
        assert self.epub_file.suffix == ".epub", f"{self.epub_file} Must be epub file"
        book_name = self.epub_file.name.split(".")[0]
        self.book_name = book_name
        book_path = Path(".epub_temp_dir") / Path(book_name)
        if not Path(".epub_temp_dir").exists():
            os.mkdir(".epub_temp_dir")
        with zipfile.ZipFile(self.epub_file) as f:
            f.extractall(book_path)
        self.book_path = book_path

    @staticmethod
    def _add_stylesheet_to_html(html_file_path, stylesheet_line):
        with open(html_file_path, "r", encoding="utf-8", errors="ignore") as file:
            content = file.read()

        soup = bs(content, "html.parser")

        # Find the head section or create if not present
        head = soup.find("head")
        if not head:
            head = soup.new_tag("head")
            soup.html.insert(0, head)

        # Add the stylesheet line inside the head section
        head.append(bs(stylesheet_line, "html.parser").contents[0])

        with open(html_file_path, "w", encoding="utf-8", errors="ignore") as file:
            file.write(str(soup))

    def _make_epub_values(self):
        """
        setups:
          1. extract the epub files
          2. make the file dict
          3. find the key file -> opf file
          4. find if has css file and make all css files to list
        """
        self.extract_one_epub_to_dir()
        self.files_dict = _make_epub_files_dict(self.book_path)
        opf_files = self.files_dict.get(".opf", [])
        assert len(opf_files) == 1, "Epub must have only one opf file"
        self.opf_file = opf_files[0]
        self.opf_dir = self.opf_file.parent.absolute()

    def change_epub_to_vertical(self):
        """
        steps:
          1. check if have CSS files
          2. check the epub spine `page-progression-direction` add to it
          3. check `primary-writing-mode` in opf file's meta, if have change it to vertical-rl, if not add it.
          4. if we have add CSS files we need to check if have `html` attribute
          5. if have `html` attribute add vertical-rl to it
          6. if have not `html` we add it
          7. if we do not have css file, we add one with html `vertical-rl` and change all the html to add the css files
        """
        soup = load_opf_meta_data(self.opf_file)
        # change it to rtl -> right to left
        spine = soup.find("spine")
        if spine.attrs.get("page-progression-direction", "") != "rtl":
            spine.attrs["page-progression-direction"] = "rtl"
        meta_list = soup.find_all("meta")
        for m in meta_list:
            if m.attrs.get("name", "") == "primary-writing-mode":
                m.attrs["content"] = "vertical-rl"
        else:
            meta_list.append(bs(V_STYLE_LINE_IN_OPF, "xml").contents[0])
        with open(self.opf_file, "w", encoding="utf-8", errors="ignore") as file:
            file.write(str(soup))

        manifest = soup.find_all("manifest")[0]
        items = [i for i in manifest.find_all("item")]
        self.css_files = [
            self.opf_dir / Path(i.attrs.get("href", ""))
            for i in items
            if i.attrs.get("media-type", "") == "text/css"
        ]
        self.has_css_file = len(self.css_files) > 0
        if self.has_css_file:
            for css in self.css_files:
                c = cssutils.CSSParser()
                p = c.parseFile(css)
                has_html_or_body = False
                for s in p.cssRules.rulesOfType(1):
                    if s.selectorText == "html":
                        has_html_or_body = True
                        for w in WRITING_KEY_LIST:
                            if w not in s.style.keys():
                                # set it to vertical
                                s.style[w] = "vertical-rl"
                if not has_html_or_body:
                    p.add(
                        "html {-epub-writing-mode: vertical-rl; writing-mode: vertical-rl; -webkit-writing-mode: vertical-rl}"
                    )
                css_style = p.cssText
                with open(css, "wb") as f:
                    f.write(css_style)
        else:
            # if we have no css file in the epub than we create one.
            style_path = Path(self.opf_dir) / Path("Style")
            if not style_path.exists():
                os.mkdir(style_path)
            new_css_file = style_path / Path("style.css")
            with open(new_css_file, "w", encoding="utf-8", errors="ignore") as f:
                f.write(
                    """
@charset "utf-8";
html {
  -epub-writing-mode: vertical-rl;
  writing-mode: vertical-rl;
  -webkit-writing-mode: vertical-rl
}
                        """
                )
            # add css item to manifest items
            soup.find_all("manifest")[0].append(
                bs(V_ITEM_TO_ADD_IN_MANIFEST, "xml").contents[0]
            )
            # then we need to change all html files
            for f in (
                self.files_dict.get(".html", [])
                + self.files_dict.get(".xhtml", [])
                + self.files_dict.get(".htm", [])
            ):
                self._add_stylesheet_to_html(f, V_STYLE_LINE)

    def change_epub_to_horizontal(self):
        """
        steps:
          1. check if have CSS files
          2. check the epub spine `page-progression-direction` add to it
          3. check `primary-writing-mode` in opf file's meta, if have change it to horizontal-rl, if not add it.
          4. check all css files and remove all "writing-mode", "-webkit-writing-mode", "-epub-writing-mode" to make it default that is horizontal
        """
        soup = load_opf_meta_data(self.opf_file)
        # change it to ltr -> left to right
        spine = soup.find("spine")
        if spine.attrs.get("page-progression-direction", "") != "ltr":
            spine.attrs["page-progression-direction"] = "ltr"
        meta_list = soup.find_all("meta")
        for m in meta_list:
            if m.attrs.get("name", "") == "primary-writing-mode":
                m.attrs["content"] = "horizontal-lr"
        else:
            meta_list.append(bs(H_STYLE_LINE_IN_OPF, "xml").contents[0])
        with open(self.opf_file, "w", encoding="utf-8", errors="ignore") as file:
            file.write(str(soup))

        manifest = soup.find_all("manifest")[0]
        items = [i for i in manifest.find_all("item")]
        self.css_files = [
            self.opf_dir / Path(i.attrs.get("href", ""))
            for i in items
            if i.attrs.get("media-type", "") == "text/css"
        ]
        self.has_css_file = len(self.css_files) > 0
        if self.has_css_file:
            for css in self.css_files:
                c = cssutils.CSSParser()
                p = c.parseFile(css)
                for s in p.cssRules.rulesOfType(1):
                    for k in s.style.keys():
                        if k in WRITING_KEY_LIST:
                            del s.style[k]
                css_style = p.cssText
                with open(css, "wb") as f:
                    f.write(css_style)

    def convert(self):
        if self.converter is None:
            return

        for html_file in (
            self.files_dict.get(".html", [])
            + self.files_dict.get(".xhtml", [])
            + self.files_dict.get(".htm", [])
        ):
            with open(html_file, "r", encoding="utf-8", errors="ignore") as f:
                content = f.read()
            soup = bs(content, "html.parser")

            html_element = soup.find("html")
            text_elements = html_element.find_all(string=True)

            for element in text_elements:
                old_text = element.string
                if old_text is not None:
                    new_text = self.converter.convert(old_text)
                    element.string.replace_with(new_text)
            html_element.replace_with(html_element)

            with open(html_file, "w", encoding="utf-8", errors="ignore") as file:
                file.write(soup.prettify())

    def pack(self, method="to_vertical"):
        lang = "original"
        if self.convert_to is not None:
            lang = self.convert_to
        if method == "to_vertical":
            book_name_v = f"{self.book_name}-v-{lang}.epub"
        else:
            book_name_v = f"{self.book_name}-h-{lang}.epub"

        shutil.make_archive(book_name_v, "zip", self.book_path)
        os.rename(book_name_v + ".zip", book_name_v)
        shutil.rmtree(self.book_path)

    def run(self, method="to_vertical"):
        assert method in [
            "to_horizontal",
            "to_vertical",
        ], "must be to_horizontal or to_vertical."
        ### make the basic epub value we need ###
        self._make_epub_values()
        if method == "to_vertical":
            self.change_epub_to_vertical()
        elif method == "to_horizontal":
            self.change_epub_to_horizontal()
        else:
            raise Exception("Only support epub to vertical or horizontal for now")

        self.convert()
        self.pack(method=method)
